
/* File: 3DMMP.cpp
 *
 * Desc: This is the mediaplayer demo main source file.  This code sets
 * up the main run loop, user input / system events, and the 'master'
 * scene, which sets up the rest of the system (scenes, cards, etc.).
 *
 * Mary had a little lamb.  Its fleece was scratchy and uncomfortable.
 *
 * Copyright  20010 Freescale Semiconductor
 */

#include <stdio.h>
#include <assert.h>
#include <string.h>
#include <fcntl.h> 
#include <malloc.h> 
#include <math.h> 
#include <stdlib.h>
#include <sys/time.h>
#include <time.h>
#include <unistd.h>
#include <limits.h>
#include <sys/select.h>

#include <GLES2/gl2.h>
#define GL_GLEXT_PROTOTYPES 1
#include <GLES2/gl2ext.h>
#include <GLES2/gl2amdext.h>
#include <EGL/egl.h>
#define EGL_EGLEXT_PROTOTYPES 1
#include <EGL/eglext.h>

#if defined(EGL_USE_X11) || defined(EGL_USE_HYBRID)

#include <X11/X.h>
#include <X11/Xlib.h>
#include <X11/Xutil.h>
#include <X11/extensions/Xcomposite.h>
#include <X11/extensions/render.h>
#include <X11/extensions/Xrender.h>

#endif

#include <FSL/fslutil.h>
#include <CARD/fslscene.h>
#include "3DMMP_masterscene.h"
#include <cstdio>
#include <iostream>

#ifdef DBGPRINT
#undef DBGPRINT
#endif

#if 1
/*#define PREPRINT std::cout << "MAIN: [" << __FILE__ << ":" << __LINE__ << "] ";*/
#define PREPRINT std::cout << "MAIN: ";
#define DBGPRINT(...) PREPRINT printf(__VA_ARGS__);
#else
#define DBGPRINT(fmt,...)
#endif

//--------------------------------------------------------------------------------------
// Types
//--------------------------------------------------------------------------------------

#if defined(EGL_USE_X11) || defined(EGL_USE_HYBRID)
//Stucture for keeping track of various X11 vars
typedef struct XWindow_s {
	Display *display;
	Window window;
	int screen;
	Visual *visual;
	GC gc;
	Colormap colormap;
	int depth;
	int pixelsize;
	XFontStruct *font_info;
	XWindowAttributes attr;
	//Picture picture;
	Pixmap pixmap;
} XWindow;
#else
typedef int XWindow;
#endif

#ifdef EGL_USE_HYBRID
//this is needed for getting a full screen window
typedef struct
                {
                unsigned long   flags;
                unsigned long   functions;
                unsigned long   decorations;
                long            inputMode;
                unsigned long   status;
                } Hints;
#endif

//--------------------------------------------------------------------------------------
// Global variables
//--------------------------------------------------------------------------------------

EGLDisplay		g_egldisplay;
EGLConfig		g_eglconfig;
EGLSurface		g_eglsurface;
EGLContext		g_eglcontext;

unsigned int	g_currentFrame = 0;

//This is the 'master scene' that builds the demo and instaniates the othere scenes
cesDemoMasterScene g_myScene( NULL );

//--------------------------------------------------------------------------------------
// Macro constants
//--------------------------------------------------------------------------------------

#define PRINT_FRAMERATE 1

//--------------------------------------------------------------------------------------
// Name: initGL
// Desc: Sets up GL variables and state before rendering.  Sets up EGLImage from X11 pixmap
//--------------------------------------------------------------------------------------
fslBool initGL( )
{
	DBGPRINT("+Starting GL Initialization: \n");

	//Default initialization of the scene
	if( g_myScene.initScene() != FSL_STATUS_SUCCESS )
		return FSL_FALSE;
	
	//Make the scene visible
	g_myScene.transitionOnStage(FSL_TRANSITION_UNSPECIFIED);
	
	DBGPRINT("-Finished GL Initialization\n");

	return FSL_TRUE;
}

//--------------------------------------------------------------------------------------
// Name: cleanupGL()
// Desc: cleanup any objects created in initGL
//--------------------------------------------------------------------------------------
void cleanupGL( )
{
	DBGPRINT("+Starting GL Cleanup: \n");
	DBGPRINT("-Finished GL Cleanup\n");	
}

//--------------------------------------------------------------------------------------
// Name: render
// Desc: Render the scene
//--------------------------------------------------------------------------------------
void render(int width, int height)
{
	//Clear the screen
	glClearColor( 0.0f, 0.0f, 0.0f, 1.0f );
	glClear( GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT );	
	
	//Render the scene to the back buffer
	g_myScene.setAspectRatio( width / height );
	g_myScene.renderScene( FSL_PHASE_UNSPECIFIED );
	
	//Swap front and back buffers
	eglSwapBuffers(g_egldisplay, g_eglsurface);
}

//--------------------------------------------------------------------------------------
// Name: initEGL()
// Desc: Initialize the X window and EGL objects / state need draw to screen
//--------------------------------------------------------------------------------------

fslBool initEGL(XWindow* xwin)
{
	static const EGLint configAttribs16[] =
	{
			EGL_RED_SIZE,           5,
			EGL_GREEN_SIZE,         6,
			EGL_BLUE_SIZE,          5,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,
			EGL_SAMPLES,            0,
			EGL_NONE
	};
	EGLint	numconfigs 	= 0;
	
#if defined(EGL_USE_X11)
	static const EGLint configAttribs24[] =
	{
			EGL_RED_SIZE,           8,
			EGL_GREEN_SIZE,         8,
			EGL_BLUE_SIZE,          8,
			EGL_ALPHA_SIZE,         0,
			EGL_DEPTH_SIZE,			16,			
			EGL_SAMPLES,            0,
			EGL_NONE
	};


	unsigned int	width		= 500;
	unsigned int	height		= 350;

	unsigned int 	dummy, depth;
	int 		x;
	Window 		rootwindow;

	DBGPRINT("+Starting Main Window X Initialization:\n");

	DBGPRINT("\tXOpenDisplay [");
	xwin->display = XOpenDisplay(NULL);
	DBGPRINT("%d]\n", (int)xwin->display);
	assert(xwin->display != NULL);

	DBGPRINT("\tDefaultScreen [");
	xwin->screen = DefaultScreen(xwin->display);
	DBGPRINT("%d]\n", xwin->screen);

	rootwindow = RootWindow(xwin->display,xwin->screen);

	DBGPRINT("\tXCreateSimpleWindow \n");
	xwin->window = XCreateSimpleWindow(xwin->display, rootwindow, 0, 0, width, height, 0, 0, WhitePixel (xwin->display, xwin->screen));
	
	XMapWindow(xwin->display, xwin->window);
	
	XStoreName(xwin->display,xwin->window,"Freescale Multimedia + Graphics Demo");

	{
		XSizeHints Hints;
		Hints.flags=PSize|PMinSize|PMaxSize;
		Hints.min_width=Hints.max_width=Hints.base_width=width;
		Hints.min_height=Hints.max_height=Hints.base_height=height;
		XSetWMNormalHints(xwin->display,xwin->window,&Hints);
	}	
	
	if(!XGetGeometry(xwin->display, xwin->window, &rootwindow, &x, &x, &width, &height, &dummy, &depth) || (depth == 0))
	{
		DBGPRINT("\tERROR: X11 Window Creation Error\n");
		return FSL_FALSE;
	}
	
	DBGPRINT("-Finished Main Window X Initialization:\n");
	
	DBGPRINT("+Starting EGL Initialization:\n");

	//Initialize EGL
	DBGPRINT("\teglGetDisplay \n");
	g_egldisplay = eglGetDisplay ( xwin->display );
	fslEGLCheck(FSL_TRUE);

	DBGPRINT("\teglInitialize \n");
	eglInitialize(g_egldisplay, NULL, NULL);
	fslEGLCheck(FSL_TRUE);

	DBGPRINT("\teglBindAPI \n");
	eglBindAPI(EGL_OPENGL_ES_API);

	DBGPRINT("\teglChooseConfig \n");
	if(depth == 16)
		eglChooseConfig(g_egldisplay, configAttribs16, &g_eglconfig, 1, &numconfigs);
	else if(depth == 24)
		eglChooseConfig(g_egldisplay, configAttribs24, &g_eglconfig, 1, &numconfigs);
	else
	{
		DBGPRINT("\tERROR: Only 16b and 24b windows supported \n");
		return FSL_FALSE;
	}
	
	fslEGLCheck(FSL_TRUE);

	if(numconfigs != 1)
	{
		DBGPRINT("\tERROR: No EGL suitable configurations found. \n");
		return FSL_FALSE;
	}

	DBGPRINT("\teglCreateWindowSurface \n");
	g_eglsurface = eglCreateWindowSurface(g_egldisplay, g_eglconfig, xwin->window, NULL);
	fslEGLCheck(FSL_TRUE);
#elif defined(EGL_USE_HYBRID)
	unsigned int	width		= 500;
	unsigned int	height		= 350;

	unsigned int 	dummy, depth;
	int 		x, y;
	Window 		rootwindow;
        Hints   	hints;
	Atom		property;
	unsigned long valuemask = CWOverrideRedirect;
	XSetWindowAttributes attributes;

	DBGPRINT("+Starting Main Window X Initialization:\n");

	DBGPRINT("\tXOpenDisplay [");
	xwin->display = XOpenDisplay(NULL);
	DBGPRINT("%d]\n", (int)xwin->display);
	assert(xwin->display != NULL);

	DBGPRINT("\tDefaultScreen [");
	xwin->screen = DefaultScreen(xwin->display);
	DBGPRINT("%d]\n", xwin->screen);

	rootwindow = RootWindow(xwin->display,xwin->screen);



	
	xwin->window = XCreateSimpleWindow(xwin->display, XDefaultRootWindow(xwin->display), 0, 0, (DisplayWidth(xwin->display, xwin->screen)), 
			(DisplayHeight(xwin->display, xwin->screen)), 0, 0, WhitePixel (xwin->display, xwin->screen));
	
	attributes.override_redirect = True;
	XChangeWindowAttributes(xwin->display, xwin->window, valuemask, &attributes);

        hints.flags = 2;        // Specify that we're changing the window decorations.
        hints.decorations = 0;  // 0 (false) means that window decorations should go bye-bye.

	property = XInternAtom(xwin->display,"_MOTIF_WM_HINTS",True);
	XChangeProperty(xwin->display,xwin->window,property,property,32,PropModeReplace,(unsigned char *)&hints,5);

	XMapWindow(xwin->display, xwin->window);
	
	XStoreName(xwin->display,xwin->window,"Freescale Multimedia + Graphics Demo");

	{
		XSizeHints Hints;
		Hints.flags=PSize|PMinSize|PMaxSize;
		Hints.min_width=Hints.max_width=Hints.base_width=width;
		Hints.min_height=Hints.max_height=Hints.base_height=height;
		XSetWMNormalHints(xwin->display,xwin->window,&Hints);
	}	
	
	if(!XGetGeometry(xwin->display, xwin->window, &rootwindow, &x, &x, &width, &height, &dummy, &depth) || (depth == 0))
	{
		DBGPRINT("\tERROR: X11 Window Creation Error\n");
		return FSL_FALSE;
	}
	
	DBGPRINT("-Finished Main Window X Initialization:\n");
	
	g_egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);

        DBGPRINT("*eglInitialize \n");
	eglInitialize(g_egldisplay, NULL, NULL);
	fslEGLCheck(FSL_TRUE);;

	DBGPRINT("*eglChooseConfig \n");
	eglChooseConfig(g_egldisplay, configAttribs16, &g_eglconfig, 1, &numconfigs);
	fslEGLCheck(FSL_TRUE);
	if(numconfigs != 1)
	{
		DBGPRINT("\tERROR: No EGL suitable configurations found. \n");
		return FSL_FALSE;
	}

	DBGPRINT("*eglCreateWindowSurface \n");
	g_eglsurface = eglCreateWindowSurface(g_egldisplay, g_eglconfig, open("/dev/fb0", O_RDWR), NULL);
	fslEGLCheck(FSL_TRUE);	
	
#else 
	g_egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);

    DBGPRINT("*eglInitialize \n");
	eglInitialize(g_egldisplay, NULL, NULL);
	fslEGLCheck(FSL_TRUE);;

	DBGPRINT("*eglChooseConfig \n");
	eglChooseConfig(g_egldisplay, configAttribs16, &g_eglconfig, 1, &numconfigs);
	fslEGLCheck(FSL_TRUE);
	if(numconfigs != 1)
	{
		DBGPRINT("\tERROR: No EGL suitable configurations found. \n");
		return FSL_FALSE;
	}

	DBGPRINT("*eglCreateWindowSurface \n");
	g_eglsurface = eglCreateWindowSurface(g_egldisplay, g_eglconfig, open("/dev/fb0", O_RDWR), NULL);
	fslEGLCheck(FSL_TRUE);	
#endif


	DBGPRINT("\tglCreateContext \n");
	g_eglcontext = eglCreateContext(g_egldisplay, g_eglconfig, NULL, NULL);
	fslEGLCheck(FSL_TRUE);

	DBGPRINT("\teglMakeCurrent \n");
	eglMakeCurrent(g_egldisplay, g_eglsurface, g_eglsurface, g_eglcontext);
	fslEGLCheck(FSL_TRUE);
	
	DBGPRINT("-Finished EGL Initialization:\n");
	
	return FSL_TRUE;
}

//--------------------------------------------------------------------------------------
// Name: cleanupEGL
// Desc: Cleanup any objects created in initEGL
//--------------------------------------------------------------------------------------

void cleanupEGL(XWindow* xwin)
{
	DBGPRINT("Cleaning Up EGL and Main Window\n");
	eglMakeCurrent(g_egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	eglTerminate(g_egldisplay);
#if defined( EGL_USE_X11) || defined (EGL_USE_HYBRID)	
    XCloseDisplay(xwin->display);
#endif	
	eglReleaseThread();
}

//--------------------------------------------------------------------------------------
// Name: main
// Desc: main
//--------------------------------------------------------------------------------------
int main (void)
{
	XWindow	xMainWindow;
	EGLint 	width = 0, height = 0;
	float	animStartTime = 0, animEndTime = 0;
	fslInputEventType inputEvent;

#if defined(EGL_USE_X11) || defined(EGL_USE_HYBRID)
	XEvent 	event;
#endif	
	
#ifdef PRINT_FRAMERATE	
	unsigned int	startTime = 0, endTime = 0;
#endif	

	
	DBGPRINT("Freescale Sample Card Demo\n");
	DBGPRINT("++Starting Initialization\n");
	
	assert( initEGL( &xMainWindow ) );
        g_myScene.setEGLDisplay(g_egldisplay);
	assert( initGL( ) );
	
	eglQuerySurface(g_egldisplay, g_eglsurface, EGL_WIDTH, &width);
	eglQuerySurface(g_egldisplay, g_eglsurface, EGL_HEIGHT, &height);
	DBGPRINT("--Finished Initializing (%d x %d).  Now Rendering. \n",width, height);

	//Set start time for animation system
	animStartTime = ((float) fslGetTickCount() / 1000.f);

#if defined(EGL_USE_X11) || defined(EGL_USE_HYBRID)

	//Register input mask with X11
	XSelectInput(xMainWindow.display,xMainWindow.window,
	       ExposureMask|KeyPressMask|KeyReleaseMask|ButtonPressMask|ButtonReleaseMask);
		   
	//Register closing window event.
	Atom wmDelete = XInternAtom(xMainWindow.display,"WM_DELETE_WINDOW",True);
	XSetWMProtocols(xMainWindow.display, xMainWindow.window, &wmDelete, 1);			   

#else

	//Set FSL touchscreen access
	int fd,rd;
	struct input_event ev[64];	
	unsigned int currentX = 0;
	unsigned int currentY = 0;
	unsigned int lastX = 0;
	unsigned int lastY = 0;	
	fslBool pointerDown = FSL_FALSE;
	fd_set fds;
	struct timeval zeroTime;
	int ret;
	
	//Polling interval
	zeroTime.tv_sec = 5;
	zeroTime.tv_usec = 0;
	
	//User usually needs to be root to access touchscreen events
	//So, we need to test to make sure we can access it
	fd = open("/dev/input/event1", O_RDONLY|O_NONBLOCK);
	if(fd < 0)
	{
		DBGPRINT("ERROR: Cannot open touchscreen input\n");
		return 0;
	}
	
	//Init touchscreen structure
	FD_ZERO(&fds);
	FD_SET(fd, &fds);
	
#endif

	if(g_myScene.isRunning())
		DBGPRINT("Scene has started\n");
		   
	//Main run loop
	while (g_myScene.isRunning())
	{
		eglQuerySurface(g_egldisplay, g_eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(g_egldisplay, g_eglsurface, EGL_HEIGHT, &height);

#if defined(EGL_USE_X11) || defined(EGL_USE_HYBRID)
		//X11 input polling code
		
		//Check for new events here
		while(XPending(xMainWindow.display))
		{
			//Grab next event off the X11 event queue
			XNextEvent(xMainWindow.display,&event);
			
			//Handle the event
			switch(event.type)
			{
			
			case ButtonPress:  //Mouse button press
				inputEvent.input = FSL_INPUT_POINTERDOWN;
				inputEvent.signalA = (unsigned int)(FSL_POINTER_MAX * ((float) event.xbutton.x / (float)width));
				inputEvent.signalB = (unsigned int)(FSL_POINTER_MAX * (((float)height - (float) event.xbutton.y) / (float)height));
				inputEvent.signalC = 0;
				inputEvent.time = event.xbutton.time;
				g_myScene.handleInputEvent( inputEvent );
				break;
				
			case ButtonRelease: //Mouse button release
				inputEvent.input = FSL_INPUT_POINTERUP;
				inputEvent.signalA = FSL_POINTER_MAX * ((float) event.xbutton.x / (float)width);
				inputEvent.signalB = FSL_POINTER_MAX * (((float)height - (float) event.xbutton.y) / (float)height);
				inputEvent.signalC = 0;
				inputEvent.time = event.xbutton.time;			
				g_myScene.handleInputEvent( inputEvent );
				break;
				
			case KeyPress: //Keyboard button press
				inputEvent.input = FSL_INPUT_KEYPRESS;
				inputEvent.signalA = event.xkey.keycode;
				inputEvent.signalB = 0;
				inputEvent.signalC = 0;
				inputEvent.time = event.xkey.time;
				g_myScene.handleInputEvent( inputEvent );
				break;
				
			case KeyRelease: //Keyboard button release
				inputEvent.input = FSL_INPUT_KEYRELEASE;
				inputEvent.signalA = event.xkey.keycode;
				inputEvent.signalB = 0;
				inputEvent.signalC = 0;
				inputEvent.time = event.xkey.time;
				g_myScene.handleInputEvent( inputEvent );
				break;

			case MotionNotify: //Mouse movement
				inputEvent.input = FSL_INPUT_POINTERMOVE;
				inputEvent.signalA = FSL_POINTER_MAX * ((float) event.xmotion.x / (float)width);
				inputEvent.signalB = FSL_POINTER_MAX * (((float) height - (float) event.xmotion.y) / (float)height);
				inputEvent.signalC = 0;
				inputEvent.time = event.xmotion.time;
				g_myScene.handleInputEvent( inputEvent );
				break;

			case ClientMessage:  //Handle window close event
				inputEvent.input = FSL_INPUT_CLOSEWINDOW;
				g_myScene.handleInputEvent( inputEvent );
				break;
				
			}
			
		}
#else
		//Touchscreen input polling
		
		//Touchscreen struct init
		FD_SET(fd, &fds);
		//Poll the touchscreen driver to see if there are any events waiting
		ret = select(fd + 1, &fds, NULL, NULL, &zeroTime);

		//If there are events ready
		if(ret)
		{
			//Read the new events
			rd = read(fd, ev, sizeof(struct input_event) * 64);

			if (rd >= (int) sizeof(struct input_event)) 
			{
				//handle the new event segment (delivered as state and ended with a sync / EV_SYN)
				for (unsigned int i = 0; i < rd / sizeof(struct input_event); i++)
				{
					switch(ev[i].type)
					{
						//End of data entry for events (sync)
						case EV_SYN:
							if(pointerDown == FSL_TRUE) 
							{
								if( (currentX == 0) && (currentY == 0) )
								{
									//Pointer Up Event
									pointerDown = FSL_FALSE;
									inputEvent.input = FSL_INPUT_POINTERUP;
									inputEvent.signalA = lastX * 10;
									inputEvent.signalB = lastY * 10;
									inputEvent.signalC = 0;
									inputEvent.time = fslGetTickCount();
									g_myScene.handleInputEvent( inputEvent );
								}
								else
								{
									//Pointer Move Event
									lastX = currentX;
									lastY = currentY;

									inputEvent.input = FSL_INPUT_POINTERMOVE;
									inputEvent.signalA = lastX * 10;
									inputEvent.signalB = lastY * 10;
									inputEvent.signalC = 0;
									inputEvent.time = fslGetTickCount();
									g_myScene.handleInputEvent( inputEvent );

								}
							}
							else
							{
								//Pointer Down Event
								pointerDown = FSL_TRUE;
								lastX = currentX;
								lastY = currentY;
								inputEvent.input = FSL_INPUT_POINTERDOWN;
								inputEvent.signalA = lastX * 10;
								inputEvent.signalB = lastY * 10;
								inputEvent.signalC = 0;
								inputEvent.time = fslGetTickCount();
								g_myScene.handleInputEvent( inputEvent );
							}
							break;
						//Data components for events
						case 3:
							//Set data state for the new event
							if(ev[i].code == 0)//X
							{
								currentX = ev[i].value;
							}
							else if(ev[i].code == 1)//Y
							{
								currentY = ev[i].value;
							}
							break;
					}
				}
			}
		}

#endif		

		//Set the scene with the new time for the animation system
		animEndTime = ( (float) fslGetTickCount() / 1000.0f);
		g_myScene.animResolve(animEndTime - animStartTime);
		animStartTime = animEndTime;
		
		//Render the screen
		render(width, height);

		//Framecounter
		g_currentFrame++;

#ifdef PRINT_FRAMERATE
		// Printing FPS metric
		if(g_currentFrame % 100 == 0)
		{
			endTime = fslGetTickCount();
			if(startTime && startTime < endTime)
			{
				
				DBGPRINT("\t%f FPS, %f ms/frame [%d] (%d x %d)\n", 
					100 * 1000.f / (endTime - startTime),  
					(float)(endTime - startTime)/(float)100, 
					g_currentFrame, width, height);
			
			}
			startTime = fslGetTickCount();
		}
#endif		
	}
	
	if(!g_myScene.isRunning())
		DBGPRINT("Scene has quit\n");
	
	cleanupGL( );
	cleanupEGL( &xMainWindow );

	return 0;
}


#undef DBGPRINT
