/* File: 3DMMP_masterscene.cpp
 * 
 * Desc: This is the 'master' scene for the media player demo.  This
 * scene instaniates the video and music player scenes, handles the input for
 * scene change overs, and passes input data that is appropriate to those scenes.
 * 
 * My name is Inigo Montoya. You killed my father. Prepare to die. 
 * 			
 * Copyright © 2009 Freescale Semiconductor
 */
 
#include "3DMMP_masterscene.h"

#include <CARD/fslvideobrowserscene.h>
#include <CARD/fslmusiccontainerscene.h>
#include <cstdio>
#include <iostream>

#ifdef DBGPRINT
#undef DBGPRINT
#endif

#if 1
/*#define PREPRINT std::cout << "MASTER: [" << __FILE__ << ":" << __LINE__ << "] ";*/
#define PREPRINT std::cout << "MASTER: ";
#define DBGPRINT(...) PREPRINT printf(__VA_ARGS__);
#else
#define DBGPRINT(fmt,...)
#endif


cesDemoMasterScene::cesDemoMasterScene( fslScene* parentScene )
{
	parentScene_ = parentScene;

	for(int i=0; i < MAX_CARDS_IN_SCENE; i++)
		cardList_[i] = NULL;

	for(int i=0; i < MAX_SCENES_IN_SCENE; i++)
		sceneList_[i] = NULL;
		
	numCards_ = 0; 
	numScenes_ = 0;
	newCardID_ = 0;
	orientation_ = FSL_LANDSCAPE;
	isVisible_ = FSL_FALSE;
	
	activeScene_ = 0;
	
	//Add the component scenes to the master scene
	addScene_( new fslVideoBrowserScene( this ) );
	addScene_( new fslMusicContainerScene( this ) );
	
	sceneTransition_ = FSL_FALSE;
	
	isRunning_ = true;
	return;
}

cesDemoMasterScene::~cesDemoMasterScene()
{
	int i=0;
	
	while(sceneList_[i] != NULL)
	{
		delete sceneList_[i];
		i++;
	}
	return;
}
	
fslStatus cesDemoMasterScene::setAspectRatio( float aspect )
{
	aspectRatio_ = aspect;
	return FSL_STATUS_SUCCESS;
}
	
fslStatus cesDemoMasterScene::setOrientation( fslOrientationType orientation )
{
	orientation_ = orientation;
	//Simple passthrough of orientation state to currently active scene
	sceneList_[activeScene_]->setOrientation( orientation );
	return FSL_STATUS_SUCCESS;
}

fslStatus cesDemoMasterScene::transitionOnStage( fslTransitionType transition)
{
	isVisible_ = FSL_TRUE;
	//Simple passthrough of transition state to currently active scene
	sceneList_[activeScene_]->transitionOnStage( transition );
	
	return FSL_STATUS_SUCCESS;
}

fslStatus cesDemoMasterScene::transitionOffStage( fslTransitionType transition)
{
	isVisible_ = FSL_FALSE;
	//Simple passthrough of transition state to currently active scene
	sceneList_[activeScene_]->transitionOffStage( transition );
	
	return FSL_STATUS_SUCCESS;
}

fslStatus cesDemoMasterScene::renderScene( fslPhaseType onlyThese)
{
	if(isVisible_ != FSL_TRUE)
		return FSL_STATUS_SUCCESS;

	//Render the currently active scene
	sceneList_[activeScene_]->setAspectRatio( aspectRatio_ );
	sceneList_[activeScene_]->renderScene( onlyThese );

	return FSL_STATUS_SUCCESS;
}

fslStatus cesDemoMasterScene::handleInputEvent( fslInputEventType event)
{
	static long lastClick = 0;
	
	//Double click to change the active scene
	if( (event.input == FSL_INPUT_POINTERDOWN)  &&  (sceneTransition_ != FSL_TRUE) 
		&& ( (event.time - lastClick) < 300 )
		)
	{
		DBGPRINT("--change scenes! \n");
		sceneList_[activeScene_]->transitionOffStage( FSL_TRANSITION_UNSPECIFIED );
		sceneTransition_ = FSL_TRUE;
		return FSL_STATUS_SUCCESS;
	}
	
	//Catch the mouse click time to detect double click
	if( event.input == FSL_INPUT_POINTERDOWN )
		lastClick = event.time;

	//Shutdown scenes on a close window event
	if (event.input == FSL_INPUT_CLOSEWINDOW || ((event.input == FSL_INPUT_POINTERUP) && (event.time - lastClick > 6000)))
	{
		for(unsigned int i = 0; i < numScenes_; i++)
			sceneList_[i]->handleInputEvent( event );
		
		isRunning_ = false;
		return FSL_STATUS_SUCCESS;
	}
	else
		//event passthrough to the active scene
		return sceneList_[activeScene_]->handleInputEvent( event );
	
	return FSL_STATUS_SUCCESS;
}

fslStatus cesDemoMasterScene::initScene()  
{
	fslStatus sceneret;

	//Initialize all of the child scenes
	for(unsigned int i = 0; i < numScenes_; i++)
	{	
		sceneList_[i]->setEGLDisplay(dpy_);
		sceneret = sceneList_[i]->initScene();

		if( sceneret != FSL_STATUS_SUCCESS )
			return sceneret;
	}
		
	return FSL_STATUS_SUCCESS;
}

fslBool cesDemoMasterScene::isOnStage()
{
	return isVisible_;
}


fslStatus cesDemoMasterScene::animResolve( float deltaTime )
{
	//Incrementing time for scene animation
	for(unsigned int i = 0; i < numScenes_; i++)
	{
		sceneList_[i]->animResolve( deltaTime );
	}
	
	//Start the animation for new scene transition on screen
	if(sceneTransition_ == FSL_TRUE)
	{
		if( sceneList_[activeScene_]->isOnStage() != FSL_TRUE )
		{
			//change to the next scene in the list (currently there are only two)
			activeScene_++;
			
			if(activeScene_ >= numScenes_)
			{
				activeScene_ = 0;
			}

			sceneList_[activeScene_]->transitionOnStage( FSL_TRANSITION_UNSPECIFIED );
			
			sceneTransition_ = FSL_FALSE;
		}
	}

	return FSL_STATUS_SUCCESS;
}

void cesDemoMasterScene::receiveCardEvent( unsigned int eventID, void* data )
{
	//There is no data for children to pass back to this scene
}

#undef DBGPRINT
