#include <FSL/fslaudiopipeline.h>

static GMainLoop *loop;    //gstreamer loop
GstElement *audioPipeline; //element containing the audio pipeline
/*elements for source file, mp3 decoder and alsasink*/
GstElement *source, *decoder, *sink; 
GstPad *pad; //gstpad for probing
float speed = 0; //floar value to grab speed from song
struct SoundInfo info; //structure to contain song information
int goomEvent = 0; //variable to keep track of audio events
bool isAudioPlaying = 0; //variable for playing control

void soundInfo_init(){ //initialization of structure values
	info.speedvar = info.accelvar = 0; 
    	info.prov_max = 0;
	info.totalgoom = 4;
	info.allTimesMax = 1;
}

//algorithm to get sound volume, speed and acceleration
//the code in the following method is based on goom by ios-software under LGLP license
//for more information see the readme file for fslaudiopipeline
struct SoundInfo evaluate_sounds(gint16 data[2][512], struct SoundInfo info) {

	int i;
	float difaccel; //acceleration delta
  	float prevspeed; //previous speed

	int incvar = 0; //max value of data
	for (i = 0; i < 512; i+=2) { //find data max value
		if (incvar < data[0][i])
			incvar = data[0][i];
	}

	//if data contains a bigger value than the previous sample set the value as allTimesMax
	if (incvar > info.allTimesMax)
		info.allTimesMax = incvar;

	//get a relative value to the allTimesMax volume
	info.volume = (float)incvar / (float)info.allTimesMax;
	//copy data to the sample arrays so the song speed is not affected
	memcpy(info.samples[0],data[0],512*sizeof(short));
	//copy data to the sample arrays so the song speed is not affected
	memcpy(info.samples[1],data[1],512*sizeof(short));

	/*get acceleration value*/
	difaccel = info.accelvar;
	info.accelvar = info.volume; 

	if (info.speedvar > 1.0f)
		info.speedvar = 1.0f;

	if (info.speedvar < 0.1f)
		info.accelvar *= (1.0f - (float)info.speedvar);
	else if (info.speedvar < 0.3f)
		info.accelvar *= (0.9f - (float)(info.speedvar-0.1f)/2.0f);
	else
		info.accelvar *= (0.8f - (float)(info.speedvar-0.3f)/4.0f);

	info.accelvar *= ACCEL_MULT;
	if (info.accelvar < 0)
		info.accelvar = 0;

	/*get acceleration delta*/
	difaccel = info.accelvar - difaccel;
	if (difaccel < 0)
		difaccel = - difaccel;

	/*save previous speed value*/
  	prevspeed = info.speedvar;
	info.speedvar = (info.speedvar + difaccel * 0.5f) / 2;
	info.speedvar *= SPEED_MULT;
  	info.speedvar = (info.speedvar + 3.0f * prevspeed) / 4.0f;
	if (info.speedvar < 0)
		info.speedvar = 0;
	if (info.speedvar > 1)
		info.speedvar = 1;

	/*save max acceleration value*/	
	if (info.accelvar > info.prov_max)
		info.prov_max = info.accelvar;

	/*if speed is incresing increment timeSinceLastGoom*/
	if(prevspeed>0.009f && info.speedvar>prevspeed)
		info.timeSinceLastGoom++;

	/*if timeSinceLastGoom is equals to totalgoom set a goomEvent*/
	if(info.timeSinceLastGoom>=info.totalgoom)
	{
		goomEvent = 1;
		info.timeSinceLastGoom = 0;
	}
	else 
		goomEvent=0;

	speed  = info.speedvar; //update speed value

	return info;
}

//gstreamer bus to handle pipeline messages
static gboolean bus_call (GstBus *bus, GstMessage *msg, gpointer user_data)
{
	switch (GST_MESSAGE_TYPE (msg))
	{
		case GST_MESSAGE_EOS: //if the end of a song is reached
		{
			//gstramer mesage to default output
			g_message ("End-of-stream");
			//let the visualizer know the song ended
			isAudioPlaying = FALSE;
			//quit gstreamer main loop
			g_main_loop_quit (loop);
		break;
		}
		case GST_MESSAGE_ERROR: //handle gstreamer error messages
		{
			gchar *debug;
			GError *err;
			gst_message_parse_error (msg, &err, &debug);
			g_free (debug);
			g_error ("%s", err->message);
			g_error_free (err);
			g_main_loop_quit (loop);
		break;
		}
		default:
		break;
	}
	return true;
}

//probing function to get song information in realtime
static gboolean get_sound_info (GstPad *pad, GstBuffer *buffer, gpointer u_data)
{	
	gint16 data[2][512]; //to contain the data
	GMutex * soundMutex = g_mutex_new(); //fron synchronization on probing
	g_mutex_lock(soundMutex);
		//get data from the decoder output
		g_memmove(data, GST_BUFFER_DATA (buffer),2*512*2);
	g_mutex_unlock(soundMutex);	
	info = evaluate_sounds(data, info); //get song info
	return true;
}

void buildPipeline(gchar *location)
{
	loop = g_main_loop_new (NULL, FALSE); //create a gstreamer main loop
	audioPipeline = gst_pipeline_new ("player"); //create a pipeline
	
	{
		GstBus *bus; //create a gstremer bus to handle pipeline messages
		//create a file source to read from file
		source  = gst_element_factory_make ("filesrc", "source");
		//create a freescale mp3 decoder
		decoder = gst_element_factory_make ("mfw_mp3decoder", "decoder");
		//create a new alsa to play sound
		sink = gst_element_factory_make ("alsasink", "sink");
		//if any of the elements failed to be created print a warning
		if (!audioPipeline || !source || !decoder || !sink)
			g_warning ("Failed to create elements!");
		//tell file source where the file to be played is
		if (location)
		{
			//set location property of file source
			g_object_set (G_OBJECT (source), "location", location, NULL);
		}
		//set bus as audiopipeline bus
		bus = gst_pipeline_get_bus (GST_PIPELINE (audioPipeline));
		//handle bus messages in bus_call
		gst_bus_add_watch (bus, bus_call, NULL);
		gst_object_unref (bus);
		//add pipeline elements together
		gst_bin_add_many (GST_BIN (audioPipeline), source, decoder, sink, NULL);
		//link elements 
		if (!gst_element_link_many (source, decoder, sink, NULL)) { 
			g_warning ("Failed to link elements!");
		}
		//get decoder output pad
		pad = gst_element_get_pad(decoder,"sink");
		//add probe to decoder output
		gst_pad_add_buffer_probe(pad, G_CALLBACK(get_sound_info),NULL);
		gst_object_unref(pad); //unreference pad
	}
}

void play()
{
	g_main_loop_run (loop);
	gst_element_set_state (GST_ELEMENT (audioPipeline), GST_STATE_READY);
}
