// Copyright (C) 2010, Freescale Semiconductor, Inc. All Rights Reserved.
// fslmusictilecard is a simple card that represents a CD case
// The card includes two pass rendering.  1 pass is for rendering the case
// The second pass renders the case reflection.

#include <iostream>
#include <FSL/fslutil.h>
#include <CARD/fslcard.h>
#include <CARD/fslscene.h>
#include <CARD/fslmusictilecard.h>

#define MAX_SHADER_LENGTH 2000

#ifdef DBGPRINT
#undef DBGPRINT
#endif

#if 1
#define PREPRINT std::cout << "MUSIC_TILE: ";
#define DBGPRINT(...) PREPRINT printf(__VA_ARGS__);
#else
#define DBGPRINT(fmt,...)
#endif


static const char* g_strEnd =
    "}											\n";

// The shaders for this card are simple and take an alpha attribute that
// allows the application to set a global transparency 
static const char* g_strVSProgramA = 
    "uniform   mat4 gc_matModelView;							\n"
    "uniform   mat4 gc_matRotation;							\n"
    "											\n"
    "attribute vec4 gc_vVertex;								\n"
    "attribute vec4 gc_vTexcoord;							\n"
    "attribute float gc_Alpha;								\n"
    "											\n"
    "varying   vec4 gc_vVSTexcoord;							\n"
    "varying   float gc_VSAlpha;							\n"
    "											\n";

static const char* g_strVSProgramB = 
    "void main()									\n"
    "{											\n"
    "    vec4 cardPosition = gc_matRotation * gc_matModelView * gc_vVertex;		\n"	
    "    gc_vVSTexcoord = gc_vTexcoord;							\n"
    "    gc_VSAlpha = gc_Alpha;								\n";
		

static const char* g_strFSProgramA = 
    "#ifdef GL_FRAGMENT_PRECISION_HIGH							\n"
    "   precision highp float;								\n"
    "#else										\n"
    "   precision mediump float;							\n"
    "#endif										\n"
    "											\n"
    "uniform   sampler2D gc_sImageTexture;						\n"
    "varying   vec4      gc_vVSTexcoord;						\n"
    "varying    float     gc_VSAlpha;							\n"
    "											\n";

static const char* g_strFSProgramB = 
    "void main()									\n"
    "{											\n"
    "    vec4 cardColor = texture2D(gc_sImageTexture, gc_vVSTexcoord.xy);		\n"
    "    cardColor.a = gc_VSAlpha;							\n";	

//Base constructor which does nothing but initialize variables
fslMusicTileCard::fslMusicTileCard()
:isReflected_(0),
vertexLoc_(0),
texcoordLoc_(1),
vertexAlphaLoc_(2)
{
}

//Overloaded constructor which initializes variables and set the
//parentscene variable
fslMusicTileCard::fslMusicTileCard(void* parentscene)
:isReflected_(0),
vertexLoc_(0),
texcoordLoc_(1),
vertexAlphaLoc_(2)
{
	parentScene_ = parentscene;
}
//Destructor
fslMusicTileCard::~fslMusicTileCard()
{
}

//This is a simple setter function that allows the card owner to set the 
//path to the data file that contains the texture to apply to the card.
fslStatus fslMusicTileCard::setTextureFileName(char* strFilename, unsigned int len)
{
		unsigned limit = len < (FSL_UTIL_MAX_FILE_NAME_LENGTH - 1)? len : FSL_UTIL_MAX_FILE_NAME_LENGTH- 1; 
                //Probably needs some error checking
                strncat(textureFileName_, strFilename, limit);
                return FSL_STATUS_SUCCESS;
}

//A setter function to turn CD case reflection rendering
// off
fslStatus fslMusicTileCard::DisableReflection()
{
           isReflected_ = 0;
           return FSL_STATUS_SUCCESS;
}

//A setter function to turn CD case reflection rendering
// on
fslStatus fslMusicTileCard::EnableReflection()
{
           isReflected_ = 1;
           return FSL_STATUS_SUCCESS;
}


//This init function may be called multiple times to initialize
// different aspects of the OpenGLES state
fslStatus fslMusicTileCard::initCard( fslInitStageType init, unsigned int config) 
{
	static GLuint hVertexShader;
	static GLuint hFragmentShader;
        fslStatus retVal = FSL_STATUS_SUCCESS;

	switch(init)
	{
		case FSL_INIT_COMPILE:
			// Create the shader program needed to render the texture
			{
				GLint nCompileResult;
				char vs[MAX_SHADER_LENGTH];
                                char ps[MAX_SHADER_LENGTH];
				const char* pvs = (char*)(vs); // Required for C++
				const char* pps = (char*)(ps); // Required for C++ 


				//This concatenates all of the sections of the shaders into a single string to be compiled
				sprintf(ps, "%s %s %s %s %s", g_strFSProgramA, pixelPreString_, g_strFSProgramB, pixelPostString_, g_strEnd);
				sprintf(vs, "%s %s %s %s %s", g_strVSProgramA, vertexPreString_, g_strVSProgramB, vertexPostString_, g_strEnd);
				
				// Load the CD case texture from file into the texture object 
				if(!fslInit2DCTESTextureGL(textureFileName_ , &textureHandle_))
                                {
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
                                }

				DBGPRINT("Vertex Shader = \n %s \n", vs);
				DBGPRINT("Pixel Shader = \n %s \n", ps);

				// Compile the shaders
				DBGPRINT("Compiling Shaders... \n");
				DBGPRINT("...VERTEX \n");
				hVertexShader = glCreateShader( GL_VERTEX_SHADER );
				glShaderSource( hVertexShader, 1, &pvs, NULL );
				glCompileShader( hVertexShader );

				//Check for compile errors.  If any occurred
				//print them.
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					DBGPRINT("%s\n", Log);
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				DBGPRINT("...FRAGMENT \n");
				hFragmentShader = glCreateShader( GL_FRAGMENT_SHADER );
				glShaderSource( hFragmentShader, 1, &pps, NULL );
				glCompileShader( hFragmentShader );

				// Check for compile success
				nCompileResult = 0;
				glGetShaderiv(hFragmentShader, GL_COMPILE_STATUS, &nCompileResult);
				
				//Check for compile errors.  If any occurred
				//print them.
				if (!nCompileResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error compiling shader! \n");
					glGetShaderInfoLog(hFragmentShader, 1024, &nLength, Log);
					DBGPRINT("%s\n", Log);
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				// Attach the individual shaders to the common shader program
				shaderProgram_[config] = glCreateProgram();
				glAttachShader( shaderProgram_[config], hVertexShader );
				glAttachShader( shaderProgram_[config], hFragmentShader );

				// Init attributes BEFORE linking
				glBindAttribLocation(shaderProgram_[config], vertexLoc_, "gc_vVertex");
				glBindAttribLocation(shaderProgram_[config], texcoordLoc_, "gc_vTexcoord");
				glBindAttribLocation(shaderProgram_[config], vertexAlphaLoc_, "gc_Alpha");
				
				numShaderAttributes_ = 3; //The scene can use attributes higher than this

			}
 		
			break;
		case FSL_INIT_LINK:
			{
				GLint nLinkResult;
				
				// Link the vertex shader and fragment shader together
				DBGPRINT("Linking Shaders... \n");
				glLinkProgram( shaderProgram_[config] );

				// Check for link success
				nLinkResult = 0;
				glGetProgramiv(shaderProgram_[config], GL_LINK_STATUS, &nLinkResult);
				// If thee were errors, print them.
				if (!nLinkResult)
				{
					char Log[1024];
					GLint nLength;
					DBGPRINT("Error linking shader! \n");
					glGetProgramInfoLog(shaderProgram_[config], 1024, &nLength, Log);
					DBGPRINT("%s\n", Log);
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
				}

				//Query the identifier for the Uniforms in the shaders
				modelMatrixLoc_[config] = glGetUniformLocation( shaderProgram_[config], "gc_matModelView" );
                                if(modelMatrixLoc_[config] == -1)
                                {
					retVal = FSL_STATUS_GENERAL_ERROR;
                                        break;
                                }

				glDeleteShader( hVertexShader );
				glDeleteShader( hFragmentShader );
				DBGPRINT("Finished Initialization... \n");	
			}			
			break;
		default:
				retVal = FSL_STATUS_BAD_PARAMETER;
                                break;
	}
	
	return retVal;
}

//Releases dynamic resources without destroying card
fslStatus fslMusicTileCard::shutdownCard() 
{
	//Nothing implemented here
	return FSL_STATUS_SUCCESS;
}

//Reallocates resources for card; probably the same as initCard for most cards
fslStatus fslMusicTileCard::resumeCard()   
{
	//Nothing implemented here
	return FSL_STATUS_SUCCESS;
}

//This card does not respond to input events
fslStatus fslMusicTileCard::handleInputEvent(fslInputEventType)
{

        return FSL_STATUS_SUCCESS;
}

//This card has no internal animation
fslStatus fslMusicTileCard::animTick(void) 
{
	return FSL_STATUS_SUCCESS;
}

//This card has no internal animation
fslStatus fslMusicTileCard::setTickRes(float time)
{
	tickTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

//This card has no internal animation
fslStatus fslMusicTileCard::setTime(float time)
{
	currentTime_ = time;
	
	return FSL_STATUS_SUCCESS;
}

//This card currently has no animation
float fslMusicTileCard::getTime()
{
	return currentTime_;
}

//This function does that actual work of drawing the card related graphics elements
fslStatus fslMusicTileCard::renderCard(unsigned int phaseID, unsigned int config)  
{
	switch(phaseID)
	{
		//Non transparent pass.  This is where the actual case is drawn.
		case 1:
                {        
			float bottom = -1.0f;
			float left = 1.0f;
			float x = -1.0f;

			//If the reflection is going to be drawn the coordinates
			// of the actual case have to be adjusted to allow space
			//for the reflection.
                        if (isReflected_)
			{
				bottom = 0.0f;
				left = 0.5f;
				x = -0.5f;
			}

			//setup the coord array for the triangles that represent
			// the case
			float VertexPositions[] =
								{
									x,	bottom , 0.0f, 1.0f,
									left,	bottom, 0.0f, 1.0f,
									x,	1.0f, 0.0f, 1.0f,
									left,	1.0f, 0.0f, 1.0f
								};

			//Set coordinates for texture to be applied to vertices
			// values are float in the range [0,1]
			float VertexTexcoord[] =
								{
									2.0f / 512.0f,	398.0/512.0f,
									400.0f / 512.0f,	398.0f/512.0f,
									2.0f / 512.0f,	2.0f / 512.0,
									400.0f / 512.0f,	2.0f / 512.0
								};

			// Set the texture, texture and vertex arrays
			glBindTexture( GL_TEXTURE_2D, textureHandle_ );

			glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
			glEnableVertexAttribArray( vertexLoc_ );

			glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexcoord);
			glEnableVertexAttribArray( texcoordLoc_ );

			//Do the actual drawing
			glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );

			glDisableVertexAttribArray( vertexLoc_ );
			glDisableVertexAttribArray( texcoordLoc_ );
                }
			break;
		//Transparent pass.  This is where the case reflection is drawn.
		case 2: 
                        if (isReflected_)
                        {
				//setup an alpha gradient from 50% transparent to 100% transparent
			    float vertexAlpha[4] = {0.5f, 0.5f, 0.0f, 0.0f};

				//Setup vetex coordinates
			    float VertexPositions[] =
								{
									-.5f,	0.0f , 0.0f, 1.0f,
									0.5f,	0.0f, 0.0f, 1.0f,
									-.5,	-0.6f, 0.0f, 1.0f,
									0.5f,	-0.6f, 0.0f, 1.0f
								};

				//Setup texture coordinates see explanation from first pass
			    float VertexTexcoord[] =
								{
									2.0f / 512.0f,	398.0/512.0f,
									400.0f / 512.0f,	398.0f/512.0f,
									2.0f / 512.0f,	2.0f / 512.0,
									400.0f / 512.0f,	2.0f / 512.0
								};


				// Set the texture, texture and vertex arrays
			    glBindTexture( GL_TEXTURE_2D, textureHandle_ );

			    glVertexAttribPointer( vertexLoc_, 4, GL_FLOAT, 0, 0, VertexPositions );
			    glEnableVertexAttribArray( vertexLoc_ );

			    glVertexAttribPointer( texcoordLoc_, 2, GL_FLOAT, 0, 0, VertexTexcoord);
			    glEnableVertexAttribArray( texcoordLoc_ );

			    glVertexAttribPointer( vertexAlphaLoc_, 1, GL_FLOAT, 0, 0, vertexAlpha);
			    glEnableVertexAttribArray( vertexAlphaLoc_);

				//turn on an configure alpha blending we assume nothing about state
			    glEnable(GL_BLEND);
                            glBlendFunc(GL_SRC_ALPHA, GL_ONE_MINUS_SRC_ALPHA);
			    glDrawArrays( GL_TRIANGLE_STRIP, 0, 4 );
				//turn alpha blending back off 
			    glDisable(GL_BLEND);
			    glDisableVertexAttribArray( vertexLoc_ );
			    glDisableVertexAttribArray( texcoordLoc_ );
			    glDisableVertexAttribArray( vertexAlphaLoc_);


                        }
			break;

		default:
			return FSL_STATUS_BAD_PARAMETER;
	}
	
	return FSL_STATUS_SUCCESS;
}

//This function allows the parent to query how many render phases this 
// card supports.  End of array marked with FSL_PHASE_UNSPECIFIED
void fslMusicTileCard::getRenderPhases(fslPhaseElementType* phaseArray) 
{
	if(phaseArray == NULL)
		return;
	
	//Scene runs through array until FSL_PHASE_UNSPECIFIED is reached
	phaseArray[0].type = FSL_PHASE_RENDER_WRITEONLY;
	phaseArray[0].phaseID = 1;
	phaseArray[1].type = FSL_PHASE_RENDER_READWRITE;
	phaseArray[1].phaseID = 2;
	phaseArray[2].type = FSL_PHASE_UNSPECIFIED;
	phaseArray[2].phaseID = 0;

	return;
}

//Allows the card to give information about the aspect ratio of this card
float fslMusicTileCard::getAspectRatio()
{
	return  1.0f; 
}







