/* File: fslvideobrowserscene.cpp
 * 
 * Desc: This scene allows the user to browse, playback, otherwise control
 * the playback of video files. This scene includes the file browser card,
 * the video render card, the video chapter selection card, and the 
 * playback control card.
 *
 * Human sacrifice, dogs and cats living together... mass hysteria!
 * 			
 * Copyright 2009 Freescale Semiconductor
 */
 
 
#include <cstdio>
#include <iostream>
#include <CARD/fslscene.h>
#include <CARD/fslvideobrowserscene.h>

#ifdef DBGPRINT
#undef DBGPRINT
#endif

#if 1
/*#define PREPRINT std::cout << "VIDEOBROWSER: [" << __FILE__ << ":" << __LINE__ << "] ";*/
#define PREPRINT std::cout << "VIDEOBROWSER: ";
#define DBGPRINT(...) PREPRINT printf(__VA_ARGS__);
#else
#define DBGPRINT(fmt,...)
#endif

#define FSL_CHAPTERSELECT
#define FSL_VIDEORENDER
#define FSL_PLAYBACKCONTROL
#define FSL_FILESELECT

float g_transSpeed = 1.8f;

float scene_matProjMatrix[16] = {0};


//These are shader strings that are attached to the shaders used in the child cards

//Vertex Shader Prestring for setting up variables used in scene shader code
static char* g_strVSPre =  (char*)
		"	uniform   mat4 scene_matProj;			\n"
		"	uniform   mat4 scene_matModel;			\n";
//Vertex Shader Poststring to apply scene position transforms to the card geometry		
static char* g_strVSPost = (char*)
		"    cardPosition  = scene_matModel * cardPosition;		\n"
		"    gl_Position  = scene_matProj * cardPosition;		\n";
							
//Pixel Shader Prestring (no new vars are needed)
static char* g_strPSPre =  (char*) "								\n";
//Pixel Shader Poststrintg for basic card display (no color transforms needed)
static char* g_strPSPost = (char*) "    gl_FragColor = cardColor;				\n";
//Pixel Shader Poststrintg for displaying the reflection of a card
static char* g_strPSPost_reflection = (char*)
		"    cardColor *= 0.5;						\n"  //faux-transparency for the reflection
		"    gl_FragColor = cardColor;				\n"; //the background is assumed black


fslVideoBrowserScene::fslVideoBrowserScene( fslScene* parentScene )
{
	parentScene_ = parentScene;

	for(int i=0; i < MAX_CARDS_IN_SCENE; i++)
		cardList_[i] = NULL;

	for(int i=0; i < MAX_SCENES_IN_SCENE; i++)
		sceneList_[i] = NULL;
		
	for(int i = 0; i<16;i++)
		projMatrix_[i]=0.0;

	//Always Init these to zero; incremented by the addCard_ and addScene_ methods
	numCards_ = 0; 
	numScenes_ = 0;
	
	newCardID_ = 0;
	orientation_ = FSL_LANDSCAPE;
	isVisible_ = FSL_FALSE;

#ifdef FSL_VIDEORENDER		
	myVideoCard_.setParentScene(this);
	//1000 is the error offset for receiving events
	myVideoCard_.setBaseEventID( 1000 ); 
	addCard_( &myVideoCard_ );
#endif	

#ifdef FSL_CHAPTERSELECT	
	myChapterCard_.setParentScene(this);
	myChapterCard_.setBaseEventID( 2000 ); 
	addCard_( &myChapterCard_ );
#endif	

#ifdef FSL_PLAYBACKCONTROL
	myPlaybackCard_.setParentScene(this);
	myPlaybackCard_.setBaseEventID( 3000 ); 
	addCard_( &myPlaybackCard_ );
#endif	

#ifdef FSL_FILESELECT
	myFileBrowserCard_.setParentScene(this);
	myFileBrowserCard_.setBaseEventID( 4000 ); 
	addCard_( &myFileBrowserCard_ );
#endif	
	
	//State for tracking if the matrix has changed
	modelMatrixVideoDirty_ = FSL_TRUE;
	modelMatrixControlDirty_ = FSL_TRUE;
	modelMatrixChapterDirty_ = FSL_TRUE;
	
	fslLoadIdentityMatrix4x4(modelMatrixVideo_);
	fslLoadIdentityMatrix4x4(modelMatrixControl_);
	fslLoadIdentityMatrix4x4(modelMatrixChapter_);
	
	//pointerDownVideo_ is used for tracking video plane state transitions
	pointerDownVideo_ = FSL_FALSE;
	isTransitionOffStage_ = FSL_FALSE;
	isTransitionOnStage_ = FSL_FALSE;
	isRunning_ = true;
	return;
}

fslVideoBrowserScene::~fslVideoBrowserScene()
{
	//No dynamic allocations in this scene
	return;
}
	
fslStatus fslVideoBrowserScene::setAspectRatio( float aspect )
{
	//There is no big impact to this card for changing the aspect ratio
	aspectRatio_ = aspect;
	return FSL_STATUS_SUCCESS;
}
	
fslStatus fslVideoBrowserScene::setOrientation( fslOrientationType orientation )
{
	//There is no big impact to this card for changing the orientation
	orientation_ = orientation;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoBrowserScene::transitionOnStage( fslTransitionType transition)
{
	//Enable rendering
	isVisible_ = FSL_TRUE;
	//Start the transition animation
	isTransitionOffStage_ = FSL_FALSE;
	isTransitionOnStage_ = FSL_TRUE;
	DBGPRINT("Starting on stage animation\n")
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoBrowserScene::transitionOffStage( fslTransitionType transition)
{
	DBGPRINT("Starting off stage animation\n")
	//Start the transition animation
	isTransitionOnStage_ = FSL_FALSE;
	isTransitionOffStage_ = FSL_TRUE;
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoBrowserScene::renderScene( fslPhaseType onlyThese)
{
	float reflectionMatrix_[16];  //matrix for flip
	float reflectionVideoMatrix_[16];  //result of flip and model for video card
	
	if(isVisible_ != FSL_TRUE)
		return FSL_STATUS_SUCCESS;
		
	glDisable(GL_DEPTH_TEST);

	if(onlyThese == FSL_PHASE_RENDER_WRITEONLY || onlyThese == FSL_PHASE_UNSPECIFIED)
	{
		updateModelMatrices();
		
#ifdef FSL_VIDEORENDER			
		//Render the video player card
		myVideoCard_.preRenderCard(phases_[0], 0);
		glUniformMatrix4fv( projMatrixLoc_[0], 1, 0, projMatrix_ );
		glUniformMatrix4fv( modelMatrixLoc_[0], 1, 0, modelMatrixVideo_ );
		myVideoCard_.renderCard(phases_[0], 0);
		
		//set up the matrices for the video reflection
		fslLoadIdentityMatrix4x4(reflectionMatrix_);	
		fslRotateMatrix4x4(reflectionMatrix_,180,FSL_X_AXIS);
		fslMultMatrix4x4( reflectionVideoMatrix_, reflectionMatrix_, modelMatrixVideo_);
		
		//Add additional transforms to the reflection is the on/off stage state is set
		if(isTransitionOffStage_ == FSL_TRUE)
			fslTranslateMatrix4x4(reflectionVideoMatrix_,0,1.0 + (stageTransitionPosition_ * (g_transSpeed * 1.3) ),0);
		else if(isTransitionOnStage_ == FSL_TRUE)
			fslTranslateMatrix4x4(reflectionVideoMatrix_,0,1.0 + ((1.0f-stageTransitionPosition_) * (g_transSpeed * 1.3)),0);
		else 
			fslTranslateMatrix4x4(reflectionVideoMatrix_,0,1.0,0);
			
		//Render the reflection of the video player card
		myVideoCard_.preRenderCard(phases_[0], 1);
		glUniformMatrix4fv( projMatrixLoc_[3], 1, 0, projMatrix_ );
		glUniformMatrix4fv( modelMatrixLoc_[3], 1, 0, reflectionVideoMatrix_ );
		myVideoCard_.renderCard(phases_[0], 1);		
#endif		


#ifdef FSL_CHAPTERSELECT		
		//render the video chapter selection card
		myChapterCard_.preRenderCard(phases_[0], 0);
		glUniformMatrix4fv( projMatrixLoc_[1], 1, 0, projMatrix_ );
		glUniformMatrix4fv( modelMatrixLoc_[1], 1, 0, modelMatrixChapter_ );
		myChapterCard_.renderCard(phases_[0], 0);
#endif

#ifdef FSL_PLAYBACKCONTROL
		//render the plaback control buttons card
		myPlaybackCard_.preRenderCard(phases_[0], 0);
		glUniformMatrix4fv( projMatrixLoc_[2], 1, 0, projMatrix_ );
		glUniformMatrix4fv( modelMatrixLoc_[2], 1, 0, modelMatrixControl_ );
		myPlaybackCard_.renderCard(phases_[0], 0);
#endif

#ifdef FSL_FILESELECT
		//render the file browser card
		myFileBrowserCard_.preRenderCard(phases_[0], 0);
		glUniformMatrix4fv( projMatrixLoc_[4], 1, 0, projMatrix_ );
		glUniformMatrix4fv( modelMatrixLoc_[4], 1, 0, modelMatrixSelect_ );
		myFileBrowserCard_.renderCard(phases_[0], 0);
#endif
	
	}
	
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoBrowserScene::handleInputEvent( fslInputEventType event)
{
	float vecA[4] = {0};
	float vecC[4] = {0};
	
	DBGPRINT("[Input] New user input \n")
	
	if(isChangingState_ == FSL_TRUE)
	{
		DBGPRINT("[Input] Click discarded during animation \n")
		return FSL_STATUS_SUCCESS;
	}
	
	//Convert click coords to -1 to 1 X,Y 3D coords
	vecA[0] = ((float) event.signalA * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[1] = ((float) event.signalB * 2.0f / (float) FSL_POINTER_MAX) - 1.0f;
	vecA[2] = -1.0f; 
	vecA[3] = 1.0f; 
	
	if( (event.input == FSL_INPUT_POINTERDOWN) || (event.input == FSL_INPUT_POINTERUP) )
	{
	
		DBGPRINT("[Input] Pointer event! \n")
	
		if( (currentState_ == FSL_STATE_VIDEO_CHAPTERS) || (currentState_ == FSL_STATE_VIDEO_SELECT) )
		{
			//Playback control input detection
		
			float controlMatrix[16];
			float controlMatrixInv[16];	
			
			//Multiply input coord against inverse trandform matrices
			fslMultMatrix4x4( controlMatrix, projMatrix_, modelMatrixControl_);
			fslInvertMatrix4x4( controlMatrix, controlMatrixInv);
			fslMultMatrix4x4Vec4x1 ( controlMatrixInv, vecA, vecC );
			
			//Check bounds for intersection
			if( (vecC[0] <= 1.0) && (vecC[0] >= -1.0)
				&& (vecC[1] <= 0.7) && (vecC[1] >= 0.4)) //HACK
			{
				DBGPRINT("[Input] Playback hit detected!\n")
				event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
				event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX); 
				//Pass input to the card
				myPlaybackCard_.handleInputEvent( event ); 
				return FSL_STATUS_SUCCESS;
			}
		}

		if( currentState_ == FSL_STATE_VIDEO_SELECT )
		{		
			//Filebrowser input detection
			
			float selectMatrix[16];
			float selectMatrixInv[16];
			
			//Multiply input coord against inverse trandform matrices
			fslMultMatrix4x4( selectMatrix, projMatrix_, modelMatrixSelect_);
			fslInvertMatrix4x4( selectMatrix, selectMatrixInv);
			fslMultMatrix4x4Vec4x1 ( selectMatrixInv, vecA, vecC );
			
			//Check bounds for intersection
			if( (vecC[0] <= 2.0) && (vecC[0] >= 0.0)
				&& (vecC[1] <= 1.0) && (vecC[1] >= -1.0)) //HACK
			{
				DBGPRINT("[Input] File select hit detected!\n");
				event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
				event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
				//Pass input to the card				
				myFileBrowserCard_.handleInputEvent( event ); 
				return FSL_STATUS_SUCCESS;
			}
		}

		if( currentState_ == FSL_STATE_VIDEO_CHAPTERS )		
		{
			//Chapter selection input detection
			
			float chapterMatrix[16];
			float chapterMatrixInv[16];
			
			//Multiply input coord against inverse trandform matrices
			fslMultMatrix4x4( chapterMatrix, projMatrix_, modelMatrixChapter_);
			fslInvertMatrix4x4( chapterMatrix, chapterMatrixInv);
			fslMultMatrix4x4Vec4x1 ( chapterMatrixInv, vecA, vecC );
			
			//Check bounds for intersection
			if( (vecC[0] <= -0.1) && (vecC[0] >= -0.3)
				&& (vecC[1] <= 0.45) && (vecC[1] >= -0.45)) //HACK
			{
				vecC[1] *=2.0;
				vecC[0] = (vecC[0] * 10.0) + 2.0 ;
				
				DBGPRINT("[Input] Chapter chooser hit detected!\n")
				event.signalA = (unsigned int)(((vecC[0] + 1.0f) / 2.0f) * FSL_POINTER_MAX);
				event.signalB = (unsigned int)(((vecC[1] + 1.0f) / 2.0f) * FSL_POINTER_MAX); 
				//Pass input to the card
				myChapterCard_.handleInputEvent( event ); 
				return FSL_STATUS_SUCCESS;
			}
		}
	
	
		int viewport[4];
		float winX,winY,winZ;
		float nearX,nearY,nearZ;
		float farX,farY,farZ;			
		
		glGetIntegerv(GL_VIEWPORT,viewport);

		//Video plane input detection
		
		if(event.input == FSL_INPUT_POINTERDOWN)
		{
			winX =(float) event.signalA/20; //hack to get the raw coords
			winY = viewport[3]-((float)event.signalB*viewport[3])/FSL_POINTER_MAX; //hack to get the raws coords
			winY = (float)viewport[3]-winY;
			winZ = 0.0; //set the Z =0 so we get the nearplane
			
			//pass the screen coords and the modelview matrix of the card you want to test against with
			if(!fslUnProject(winX,winY,winZ,modelMatrixVideo_,projMatrix_,viewport,&nearX,&nearY,&nearZ))
			{
				DBGPRINT("error unprojecting!!! \n")
			}
				
			winZ = 1.0;
			if(!fslUnProject(winX,winY,winZ,modelMatrixVideo_,projMatrix_,viewport,&farX,&farY,&farZ))
			{
				DBGPRINT("error unprojecting!!! \n")
			}
				
			if(myVideoCard_.intersects(nearX,nearY,nearZ,farX,farY,farZ))
			{
				DBGPRINT("ray INTERSECTS video PLANE!!!\n")
				if(pointerDownVideo_ != FSL_TRUE)
				{
					//Set state if it intersects the video plane
					//Action will be determined on the poiter UP event
					pointerDownVideo_ = FSL_TRUE;
					pointerDownVideoTime_ = event.time;
					pointerDownVideoX_ = event.signalA;
					pointerDownVideoY_ = event.signalB;
					DBGPRINT("pX1 = %d\n",pointerDownVideoX_)
				}
			}
			else
			{
				DBGPRINT("ray DOES NOT intersect video plane\n")
			}
		}
		else 
		{
			myPlaybackCard_.handleInputEvent( event );
			myFileBrowserCard_.handleInputEvent( event );
			
			//Handle video playback card input events
			if( pointerDownVideo_ == FSL_TRUE )
			{
				DBGPRINT("Pointer UP on video\n")
				
				//reset the hit state
				pointerDownVideo_ = FSL_FALSE;
				switch( currentState_ )
				{
					case FSL_STATE_VIDEO_PLAYING:
						//if the video is playing fullscreen,
						//Detect the direction of the gesture and change state
						DBGPRINT("pX2 = %d\n",pointerDownVideoX_)
						if( ((int)event.signalA - (int)pointerDownVideoX_) > 500 )
							changeStateWithTransition(FSL_STATE_VIDEO_CHAPTERS);
						else if( ((int)event.signalA - (int)pointerDownVideoX_) < -500 )
							changeStateWithTransition(FSL_STATE_VIDEO_SELECT);
						break;
					//otherwise, just make the video fullscreen
					case FSL_STATE_VIDEO_CHAPTERS:
						changeStateWithTransition(FSL_STATE_VIDEO_PLAYING);
						break;
					case FSL_STATE_VIDEO_SELECT:
						changeStateWithTransition(FSL_STATE_VIDEO_PLAYING);
						break;
					default:
						break;
				}			
			}
		}
	}

	//handle the main window being closed
	if (event.input == FSL_INPUT_CLOSEWINDOW)
	{
#ifdef FSL_PLAYBACKCONTROL	
		myPlaybackCard_.handleInputEvent( event );
		DBGPRINT("Scene shutting down.\n)")
#endif		
		isRunning_ = false;
	}

	DBGPRINT("Returning with no action\n")
	return FSL_STATUS_SUCCESS;
}

fslStatus fslVideoBrowserScene::initScene()  //GL state inititization
{
	unsigned int i = 0;
	fslPhaseElementType phaseArray[MAX_PHASE_ARRAY_SIZE];
	
	//init the array
	for( i = 0; i < MAX_PHASE_ARRAY_SIZE; i++ )
	{
		phaseArray[i].type = FSL_PHASE_UNSPECIFIED;
		phaseArray[i].phaseID = 0;
	}
	
	//reset the index
	i=0;
	
#ifdef FSL_VIDEORENDER
	myVideoCard_.setEGLDisplay(dpy_);
	myVideoCard_.getRenderPhases( phaseArray );
	while( phaseArray[i].type != FSL_PHASE_UNSPECIFIED )
		i++;
	i++;
#endif	
		
#ifdef FSL_CHAPTERSELECT
	myChapterCard_.getRenderPhases( &phaseArray[i] );
	while( phaseArray[i].type != FSL_PHASE_UNSPECIFIED )
		i++;
	i++;
#endif	
		
#ifdef FSL_PLAYBACKCONTROL
	myPlaybackCard_.getRenderPhases( &phaseArray[2] ); 
	while( phaseArray[i].type != FSL_PHASE_UNSPECIFIED )
		i++;	
	i++;
#endif

#ifdef FSL_FILESELECT	
	myFileBrowserCard_.getRenderPhases( &phaseArray[3] ); // each of these should use a separate array
	while( phaseArray[i].type != FSL_PHASE_UNSPECIFIED )
		i++;	
	i++;
#endif

	//Set projection matrix to Identity Matrix
	fslLoadIdentityMatrix4x4(projMatrix_);

	//set perspective projection matrix
	//should be aspectRatio_, but it is hard coded for now
	fslPerspectiveMatrix4x4(projMatrix_, 50.0f, 1.3333f, 0.001f, 100.0f);
	
	//I know there is only one phase for all the cards this scene uses, but let's double check
	for( i = 0; i < MAX_PHASE_ARRAY_SIZE; i++ )
	{
		if ( (phaseArray[i].phaseID != phaseArray[0].phaseID )  && (phaseArray[i].phaseID != 0 ) )
			return FSL_STATUS_GENERAL_ERROR;
	}	
	
	//More dynamic scenes (in termes of render stages) would need to parse the whole array
	phases_[0] = phaseArray[0].phaseID; 

	

	//Set our shader contibution
#ifdef FSL_VIDEORENDER		
	myVideoCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myVideoCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	DBGPRINT("about to compile videocard shader \n")
	if( myVideoCard_.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;	
		
	if( myVideoCard_.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	shaderProgram_[0] = myVideoCard_.getShaderProgram(0);
	projMatrixLoc_[0] = glGetUniformLocation( shaderProgram_[0], "scene_matProj" );
	modelMatrixLoc_[0] = glGetUniformLocation( shaderProgram_[0], "scene_matModel" );	

	myVideoCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myVideoCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost_reflection );

	DBGPRINT("about to compile videocard reflection shader \n")
	if( myVideoCard_.initCard(FSL_INIT_COMPILE, 1) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;	
		
	if( myVideoCard_.initCard(FSL_INIT_LINK, 1) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	shaderProgram_[3] = myVideoCard_.getShaderProgram(1);
	projMatrixLoc_[3] = glGetUniformLocation( shaderProgram_[3], "scene_matProj" );
	modelMatrixLoc_[3] = glGetUniformLocation( shaderProgram_[3], "scene_matModel" );	
#endif	

#ifdef FSL_CHAPTERSELECT
	myChapterCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myChapterCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );

	DBGPRINT("about to compile chaptercard shader \n");
	if( myChapterCard_.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;	

	if( myChapterCard_.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;

	shaderProgram_[1] = myChapterCard_.getShaderProgram(0);
	projMatrixLoc_[1] = glGetUniformLocation( shaderProgram_[1], "scene_matProj" );
	modelMatrixLoc_[1] = glGetUniformLocation( shaderProgram_[1], "scene_matModel" );		
#endif	

#ifdef FSL_PLAYBACKCONTROL	
	myPlaybackCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myPlaybackCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	
	DBGPRINT("about to compile playbackcard shader \n")
	if( myPlaybackCard_.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
		
	if( myPlaybackCard_.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	shaderProgram_[2] = myPlaybackCard_.getShaderProgram(0);
	projMatrixLoc_[2] = glGetUniformLocation( (GLuint)shaderProgram_[2], "scene_matProj" );
	modelMatrixLoc_[2] = glGetUniformLocation( shaderProgram_[2], "scene_matModel" );		
#endif

#ifdef FSL_FILESELECT	
	myFileBrowserCard_.setVertexShaderAppendStrings( g_strVSPre, g_strVSPost );
	myFileBrowserCard_.setPixelShaderAppendStrings( g_strPSPre, g_strPSPost );
	
	DBGPRINT("about to compile playbackcard shader \n")
	if( myFileBrowserCard_.initCard(FSL_INIT_COMPILE, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
		
	if( myFileBrowserCard_.initCard(FSL_INIT_LINK, 0) != FSL_STATUS_SUCCESS )
		return FSL_STATUS_GENERAL_ERROR;
	
	shaderProgram_[4] = myFileBrowserCard_.getShaderProgram(0);
	projMatrixLoc_[4] = glGetUniformLocation( (GLuint)shaderProgram_[4], "scene_matProj" );
	modelMatrixLoc_[4] = glGetUniformLocation( shaderProgram_[4], "scene_matModel" );		
#endif

	changeState(FSL_STATE_VIDEO_PLAYING);
	changeStateWithTransition(FSL_STATE_VIDEO_SELECT);
	
	return FSL_STATUS_SUCCESS;
}

fslBool fslVideoBrowserScene::isOnStage()
{
	return isVisible_;
}

//This function build the model transform matrices for the cards
//based on scene state, animation time, etc.
void fslVideoBrowserScene::updateModelMatrices(void)
{
	//reset everybody
	fslLoadIdentityMatrix4x4(modelMatrixVideo_);
	fslLoadIdentityMatrix4x4(modelMatrixControl_);
	fslLoadIdentityMatrix4x4(modelMatrixChapter_);	
	fslLoadIdentityMatrix4x4(modelMatrixSelect_);		
	
	if(isChangingState_ == FSL_TRUE)  //if the scene is being animated
	{
		//Set new matrices based on what the state is transitioning too
		switch(nextState_)
		{
			case FSL_STATE_VIDEO_PLAYING:
				fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
				fslTranslateMatrix4x4(modelMatrixControl_,0,1.1 + (playbackControlPosition_ * 0.9) ,-1.0);				
				
				fslTranslateMatrix4x4(modelMatrixChapter_,-0.55 - (videoChapterPosition_ * 0.23),0,-1);
				
				//the current state affects the direction of the animation for the video player card
				switch(currentState_)
				{
					case FSL_STATE_VIDEO_CHAPTERS:
						fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1.5 + (videoRenderPosition_ * 0.5));
						fslRotateMatrix4x4(modelMatrixVideo_,-45.0f * (1 - videoRenderPosition_),FSL_Y_AXIS);	
						break;
					case FSL_STATE_VIDEO_SELECT:
						fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1.5 + (videoRenderPosition_ * 0.5));
						fslRotateMatrix4x4(modelMatrixVideo_,45.0f * (1 - videoRenderPosition_),FSL_Y_AXIS);
						break;
					default:
						break;
				}						
				
				fslScaleMatrix4x4 (modelMatrixSelect_, 0.15, 0.5, 1.0);
				fslTranslateMatrix4x4(modelMatrixSelect_,3.5 + (fileSelectionPosition_ * 2.0),0.1,-1);	
				break;				
			case FSL_STATE_VIDEO_CHAPTERS:
				fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
				fslTranslateMatrix4x4(modelMatrixControl_,0,2.0 - (playbackControlPosition_ * 0.9) ,-1.0);

				fslTranslateMatrix4x4(modelMatrixChapter_,-0.75 + (videoChapterPosition_ * 0.23),0,-1);
				
				fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1 - (videoRenderPosition_ * 0.5));
				fslRotateMatrix4x4(modelMatrixVideo_,-45.0f * videoRenderPosition_,FSL_Y_AXIS);		

				fslScaleMatrix4x4 (modelMatrixSelect_, 0.15, 0.5, 1.0);
				fslTranslateMatrix4x4(modelMatrixSelect_,3.5 + (fileSelectionPosition_ * 2.0),0.1,-1);			
				break;
				
			case FSL_STATE_VIDEO_SELECT:
				fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
				fslTranslateMatrix4x4(modelMatrixControl_,0,2.0 - (playbackControlPosition_ * 0.9) ,-1.0);

				fslTranslateMatrix4x4(modelMatrixChapter_,-0.55 - (videoChapterPosition_ * 0.23),0,-1);				
				
				fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1 - (videoRenderPosition_ * 0.5));
				fslRotateMatrix4x4(modelMatrixVideo_,45.0f * videoRenderPosition_,FSL_Y_AXIS);
				
				fslScaleMatrix4x4 (modelMatrixSelect_, 0.15, 0.5, 1.0);
				fslTranslateMatrix4x4(modelMatrixSelect_,5.5 - (fileSelectionPosition_ * 2.0),0.1,-1);					
				break;	
			default:
				break;
		}
	}
	else //static; no animation
	{
		switch(currentState_)
		{
			case FSL_STATE_VIDEO_PLAYING:
				fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
				fslTranslateMatrix4x4(modelMatrixControl_,0,2.0,-1.0);	

				fslTranslateMatrix4x4(modelMatrixChapter_,-0.78,0,-1);

				fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1);
				
				fslScaleMatrix4x4 (modelMatrixSelect_, 0.15, 0.5, 1.0);
				fslTranslateMatrix4x4(modelMatrixSelect_,5.5,0.1,-1);					
				break;
			case FSL_STATE_VIDEO_CHAPTERS:
				fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
				fslTranslateMatrix4x4(modelMatrixControl_,0,1.1,-1.0);	


				fslTranslateMatrix4x4(modelMatrixChapter_,-0.52,0,-1);

				fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1.5);
				fslRotateMatrix4x4(modelMatrixVideo_,-45.0f,FSL_Y_AXIS);	

				fslScaleMatrix4x4 (modelMatrixSelect_, 0.15, 0.5, 1.0);
				fslTranslateMatrix4x4(modelMatrixSelect_,5.5,0.1,-1);					
				break;
			case FSL_STATE_VIDEO_SELECT:
				fslScaleMatrix4x4 (modelMatrixControl_, 0.3, 0.3, 1.0);
				fslTranslateMatrix4x4(modelMatrixControl_,0,1.1,-1.0);		

				fslTranslateMatrix4x4(modelMatrixChapter_,-0.78,0,-1);
		
				fslTranslateMatrix4x4(modelMatrixVideo_,0,0,-1.5);
				fslRotateMatrix4x4(modelMatrixVideo_,45.0f,FSL_Y_AXIS);		

				fslScaleMatrix4x4 (modelMatrixSelect_, 0.15, 0.5, 1.0);
				fslTranslateMatrix4x4(modelMatrixSelect_,3.5,0.1,-1);				
				break;		
			default:
				break;				
		}	
	}

	//additional transforms for on/off stage animation
	if( isTransitionOffStage_ == FSL_TRUE )
	{
		fslTranslateMatrix4x4(modelMatrixControl_,0.0,stageTransitionPosition_ * g_transSpeed,0.0);

		fslTranslateMatrix4x4(modelMatrixChapter_,0.0,stageTransitionPosition_ * g_transSpeed,0.0);

		fslTranslateMatrix4x4(modelMatrixVideo_,0.0,stageTransitionPosition_ * g_transSpeed,0.0);

		fslTranslateMatrix4x4(modelMatrixSelect_,0.0,stageTransitionPosition_ * g_transSpeed,0.0);
	}	
	else if (isTransitionOnStage_ == FSL_TRUE)
	{
		fslTranslateMatrix4x4(modelMatrixControl_,0.0,(1.0f - stageTransitionPosition_) * g_transSpeed,0.0);

		fslTranslateMatrix4x4(modelMatrixChapter_,0.0,(1.0f - stageTransitionPosition_) * g_transSpeed,0.0);

		fslTranslateMatrix4x4(modelMatrixVideo_,0.0,(1.0f - stageTransitionPosition_) * g_transSpeed,0.0);

		fslTranslateMatrix4x4(modelMatrixSelect_,0.0,(1.0f - stageTransitionPosition_) * g_transSpeed,0.0);	
	}
}

//Set the new state without animation
void fslVideoBrowserScene::changeState(fslVideoBrowserStateType newState)
{
	currentState_ = newState;
	isChangingState_ = FSL_FALSE;
}

//Set the new state with a state transition animation
void fslVideoBrowserScene::changeStateWithTransition(fslVideoBrowserStateType newState)
{
	fslBool startTranisition = FSL_FALSE;
	
	switch(currentState_){
		case FSL_STATE_VIDEO_PLAYING:
			switch(newState){
				case FSL_STATE_VIDEO_PLAYING:

					break;
				case FSL_STATE_VIDEO_CHAPTERS:
					startTranisition = FSL_TRUE;
					break;
				case FSL_STATE_VIDEO_SELECT:
					startTranisition = FSL_TRUE;
					break;				
				default:
					break;
			}
			break;
		case FSL_STATE_VIDEO_CHAPTERS:
			switch(newState){
				case FSL_STATE_VIDEO_PLAYING:
					startTranisition = FSL_TRUE;
					break;
				case FSL_STATE_VIDEO_CHAPTERS:
					break;
				case FSL_STATE_VIDEO_SELECT:
					startTranisition = FSL_TRUE;
					break;				
				default:
					break;
			}
			break;
		case FSL_STATE_VIDEO_SELECT:
			switch(newState){
				case FSL_STATE_VIDEO_PLAYING:
					startTranisition = FSL_TRUE;
					break;
				case FSL_STATE_VIDEO_CHAPTERS:
					startTranisition = FSL_TRUE;
					break;
				case FSL_STATE_VIDEO_SELECT:
					break;				
				default:
					break;
			}
			break;
		default:
				break;
	}
	
	//Reset the animation state for the cards if we are starting a new transition animation
	if (startTranisition == FSL_TRUE)
	{
		nextState_ = newState;
		isChangingState_ = FSL_TRUE;
		videoRenderPosition_ = 0.0f;
		videoChapterPosition_ = 0.0f;
		playbackControlPosition_ = 0.0f;
		fileSelectionPosition_ = 0.0f;			
		DBGPRINT("VIDEOBROWSER: Changing state from %d to %d \n",currentState_,nextState_)
	}

}

//Set the new state if the cards have reached the ends of their transition animations
void fslVideoBrowserScene::changeStateOnPositionCheck(void)
{
	if
	( 
	( videoRenderPosition_ >= 1.0f ) &&
	( videoChapterPosition_ >= 1.0f ) &&
	( playbackControlPosition_ >= 1.0f ) &&
	( fileSelectionPosition_ >= 1.0f )
	)
	{
		currentState_ = nextState_;
		videoRenderPosition_ = 1.0f;
		videoChapterPosition_ = 1.0f;
		playbackControlPosition_ = 1.0f;
		fileSelectionPosition_ = 1.0f;							
		isChangingState_ = FSL_FALSE;
		DBGPRINT("VIDEOBROWSER: State changed to %d \n",currentState_)
	}
}

//Change the animation state an incremental amount
fslStatus fslVideoBrowserScene::animResolve( float deltaTime )
{
	//the delta time should be incorporated in this function when incrementing the card positions
	
	if(isChangingState_ == FSL_TRUE)
	{
		switch(currentState_)
		{
			case FSL_STATE_VIDEO_PLAYING:
				switch(nextState_){
					case FSL_STATE_VIDEO_PLAYING:
						isChangingState_ = FSL_FALSE;
						break;
					case FSL_STATE_VIDEO_CHAPTERS:
						videoRenderPosition_ += 0.01f;
						videoChapterPosition_ += 0.01f;
						playbackControlPosition_ += 0.01f;
						fileSelectionPosition_ = 1.0f;
						
						changeStateOnPositionCheck();
						
						break;
					case FSL_STATE_VIDEO_SELECT:
						videoRenderPosition_ += 0.01f;
						videoChapterPosition_ = 1.0f;
						playbackControlPosition_ += 0.01f;
						fileSelectionPosition_ += 0.01f;
						
						changeStateOnPositionCheck();
						
						break;				
					default:
						break;
				}
				break;
			case FSL_STATE_VIDEO_CHAPTERS:
				switch(nextState_){
					case FSL_STATE_VIDEO_PLAYING:
						videoRenderPosition_ += 0.01f;
						videoChapterPosition_ += 0.01f;
						playbackControlPosition_ += 0.01f;
						fileSelectionPosition_ = 1.0f;
						
						changeStateOnPositionCheck();
						
						break;
					case FSL_STATE_VIDEO_CHAPTERS:
						isChangingState_ = FSL_FALSE;
						break;
					case FSL_STATE_VIDEO_SELECT:
						videoRenderPosition_ += 0.01f;
						videoChapterPosition_ = 1.0f;
						playbackControlPosition_ += 1.0f;
						fileSelectionPosition_ += 0.01f;
						
						changeStateOnPositionCheck();
						
						break;				
					default:
						break;
				}
				break;
			case FSL_STATE_VIDEO_SELECT:
				switch(nextState_){
					case FSL_STATE_VIDEO_PLAYING:
						videoRenderPosition_ += 0.01f;
						videoChapterPosition_ = 1.0f;
						playbackControlPosition_ += 0.01f;
						fileSelectionPosition_ += 0.01f;
						
						changeStateOnPositionCheck();
						
						break;
					case FSL_STATE_VIDEO_CHAPTERS:
						videoRenderPosition_ += 0.01f;
						videoChapterPosition_ += 0.01f;
						playbackControlPosition_ = 1.0f;
						fileSelectionPosition_ = 1.0f;
						
						changeStateOnPositionCheck();
						
						break;
					case FSL_STATE_VIDEO_SELECT:
						isChangingState_ = FSL_FALSE;
						break;				
					default:
						break;
				}
				break;	
			default:
				break;				
		}
	}

	//Animation state for on/off stage transitions
	if( (isTransitionOffStage_ == FSL_TRUE) || (isTransitionOnStage_ == FSL_TRUE) )
	{
		stageTransitionPosition_ += 0.015;
		
		if( stageTransitionPosition_ >= 1.0 )
		{
			stageTransitionPosition_ = 0.0;
			
			if(isTransitionOffStage_ == FSL_TRUE)
			{
                               fslInputEventType event;
                               //stop the video
                               event.input = FSL_INPUT_STOP;
                               myVideoCard_.handleInputEvent( event );

				DBGPRINT("Done trans off stage, setting vis=0\n")
				isTransitionOffStage_ = FSL_FALSE;
				isVisible_ = FSL_FALSE;
			}
			else
			{
				DBGPRINT("Done trans on stage\n")
				isTransitionOnStage_ = FSL_FALSE;
			}
		}
	}
	
//resolve the card animations if they have any	
#ifdef FSL_VIDEORENDER		
	myVideoCard_.animTick();
#endif
	
#ifdef FSL_CHAPTERSELECT	
	myChapterCard_.animTick();
#endif	

#ifdef FSL_PLAYBACKCONTROL
	myPlaybackCard_.animTick();
#endif	

#ifdef FSL_FILESELECT
	myFileBrowserCard_.animTick();
#endif	

	return FSL_STATUS_SUCCESS;
}

void fslVideoBrowserScene::receiveCardEvent( unsigned int eventID, void* data )
{
	//Child Cards call this function on events
	
	fslInputEventType event;
	DBGPRINT("EventID is: %i\n",eventID)
	
	//video chapter selection events
	if((eventID >= 2000) && (eventID < 3000))
	{
		
		switch(eventID-2000)
		{
			case CH0:
				DBGPRINT("about to play chapter 0 \n")
				event.input = FSL_INPUT_SEEK;
				myVideoCard_.setChapter(CH0);
				myVideoCard_.handleInputEvent( event );		
				break;
			case CH1:
				event.input = FSL_INPUT_SEEK;
				DBGPRINT("about to play chapter 1 \n")
				myVideoCard_.setChapter(CH1);
				myVideoCard_.handleInputEvent( event );		
				break;
				
			case CH2:
				event.input = FSL_INPUT_SEEK;
				DBGPRINT("about to play chapter 2 \n")
				myVideoCard_.setChapter(CH2);
				myVideoCard_.handleInputEvent( event );		
				break;
			case CH3:
				event.input = FSL_INPUT_SEEK;
				DBGPRINT("about to play chapter 3 \n")
				myVideoCard_.setChapter(CH3);	
				myVideoCard_.handleInputEvent( event );	
				break;	
		}
	}
	
	//playback control card events
	if (eventID > 3000 && eventID < 4000)
	{
		switch(eventID-3000)
		{
			case REW:
			case STOP:
				event.input = FSL_INPUT_STOP;
				myVideoCard_.handleInputEvent( event );
				break;
			case PAUSE:
				event.input = FSL_INPUT_PAUSE;
				myVideoCard_.handleInputEvent( event );
				break;
			case PLAY:
				event.input = FSL_INPUT_PLAY;
				myVideoCard_.handleInputEvent( event  );	
				break;
			case FFWD:

			default:
				DBGPRINT("No such event\n");
		}
	
	}
	
	//file browser input events
	else if (eventID > 4000 && eventID < 5000)
	{
		DBGPRINT("Card ask for file load %s \n", (char*) data)
		myChapterCard_.setVideoFile( (char*) data );
		myVideoCard_.setVideoFile( (char*) data );
	
	}else if (eventID == FSL_INPUT_CLOSEWINDOW)
	{
		
		event.input = FSL_INPUT_CLOSEWINDOW;
		myVideoCard_.handleInputEvent( event );
		DBGPRINT("Scene shutting down.\n")
		isRunning_ = false;
	}

}

#undef DBGPRINT

