#include <stdio.h>
#include <stdlib.h>
#include <assert.h>
#include <math.h>
#include <FSL/fslutil.h>
#include <GLU3/glu3.h>
#include "meshStructures.h"
#ifndef USE_MX31
#include "GLES/gl.h"
#include "GLES/glext.h"
#include "EGL/egl.h"
#else
#include <GLES/gl.h>
#include <GLES/egl.h>
#endif
#ifdef EGL_USE_X11
#include <X11/X.h>
#include <X11/Xlib.h>
#endif

EGLDisplay			egldisplay;
EGLConfig			eglconfig;
EGLSurface			eglsurface;
EGLContext			eglcontext;

#ifdef EGL_USE_X11
Display 			*display;
#endif

int				currentFrame = 0;

//Texture handles
GLuint texture1 = 0;
GLuint texture2 = 0;

struct Mesh *mesh = NULL;

//----------------------------------------------------------------------------
struct Mesh *LoadMeshFromFile(const char *filename)
{
  struct GenericObjectData o;
  struct Mesh* mesh = (struct Mesh*) malloc(sizeof(struct Mesh));
  unsigned int i;

  struct GSDHeader header;
  FILE *meshFile = fopen(filename,"rb");
  if(!meshFile)
    return NULL;

  /*The header holds a brief description of the file, the version number, and the number of meshes
    that are stored in the file. This type of files are thought for static meshes only*/
  fread(&header,sizeof(struct GSDHeader),1,meshFile); 

  //Check if there is at least one object
  if(header.numberOfSubObjects < 1)
    return NULL;

  // we only will use the first object, so we won't iterate over the others, if they exist
  fread(o.Name,sizeof(char)*128,1,meshFile); //read the object name
  fread(o.ParentName,sizeof(char)*128,1,meshFile); //Read the name of the parent object (useful for hierarchies)
  fread(&o.iC,sizeof(unsigned long),1,meshFile); //read the number of vertex indices
  fread(&o.vC,sizeof(unsigned long),1,meshFile); //read the number of vertices

  //allocate enough space for the indices and the GLshort version of them
  o.Indices = (unsigned int*) malloc(sizeof(unsigned int)*(o.iC));
  mesh->Indices = (GLshort*) malloc(sizeof(GLshort)*(o.iC));
  fread(o.Indices, sizeof(unsigned int) * o.iC,1,meshFile); // read all indices

  //allocate enough space for the vertices
  o.Geometry = (float*) malloc(sizeof(float)*(o.vC * 3));
  mesh->Geometry = (GLfloat*) malloc(sizeof(GLfloat)*(o.vC * 3));
  fread(o.Geometry,o.vC * 3 * sizeof(float),1,meshFile); //read all vertices (1 vertex = 3 floats)

  //allocate enough space for the texture coordinates
  o.TexCoord = (float*) malloc(sizeof(float)*(o.vC * 2));
  mesh->TexCoord = (GLfloat*) malloc(sizeof(GLfloat)*(o.vC * 2));
  fread(o.TexCoord,o.vC * 2 * sizeof(float),1,meshFile);//read all texcoords (1 tex coord = 2 floats)

  //allocate enough space for the normals
  o.Normals = (float*) malloc(sizeof(float)*(o.vC * 3));
  mesh->Normals = (GLfloat*) malloc(sizeof(GLfloat)*(o.vC * 3));
  fread(o.Normals,o.vC * 3* sizeof(float),1,meshFile);//read all normals (1 normal = 3 floats)
  fclose(meshFile); //Do not need the file opened anymore

  // Convert data to optimized data types for OpenGL ES (GLfloat and GLshort)
  for(i=0;i<o.vC * 3;i++)
  {
    mesh->Geometry[i]= o.Geometry[i];
    mesh->Normals[i] = o.Normals[i];
  }

  for(i=0;i<o.vC * 2;i++)
    mesh->TexCoord[i] = o.TexCoord[i];

  for(i=0;i<o.iC;i++)
    mesh->Indices[i] = (GLshort)o.Indices[i];

  mesh->indexCounter = (GLshort)o.iC;
  mesh->vertexCounter= (GLshort)o.vC;

  //delete original values, we will use only the optimized ones
  free(o.Indices);
  free(o.Geometry);
  free(o.Normals);
  free(o.TexCoord);

  return mesh;
}

/* function to release/destroy our resources and restoring the old desktop */
void Cleanup()
{

}

/* general OpenGL initialization function */
int init(void)
{

    static const EGLint s_configAttribs[] =
    {
	EGL_RED_SIZE,		5,
	EGL_GREEN_SIZE, 	6,
	EGL_BLUE_SIZE,		5,
	EGL_ALPHA_SIZE, 	0,
      	EGL_SAMPLES, 		0,
      	EGL_NONE
    };

    EGLint numconfigs;

    #ifdef EGL_USE_X11
    //Open X display
    printf("XOpenDisplay \n");
    display = XOpenDisplay(NULL);
    assert(display != NULL);	

    //Get the default screen from display
    int screen;
    printf("DefaultScreen [");
    screen = DefaultScreen(display);
    printf("%d]\n", screen);

    //get root window
    Window window, rootwindow; 
    printf("RootWindow \n");
    rootwindow = RootWindow(display,screen); 
	
    //get egldisplay from X display
    printf("eglGetDisplay (%d)\n", (int)display);
    egldisplay = eglGetDisplay ( display);
    assert(eglGetError() == EGL_SUCCESS);

    printf("Disp = %d \n",(int) egldisplay);

    //initializa egl display
    printf("eglInitialize \n");
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);

    //tell the driver we are using OpenGL ES
    printf("eglBindAPI \n");
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    printf("eglChooseConfig \n");
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    assert(eglGetError() == EGL_SUCCESS);

    assert(numconfigs == 1);

    //Create and x window
    printf("XCreateSimpleWindow \n");
    window = XCreateSimpleWindow(display, rootwindow, 0, 0, 400, 533, 0, 0, WhitePixel (display, screen));

    XMapWindow(display, window);

    //get an eglsurface from the display using our egl configuration
    printf("*eglCreateWindowSurface \n");
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, window, NULL);
    assert(eglGetError() == EGL_SUCCESS);

    #else

    //get egl display
    egldisplay = eglGetDisplay(EGL_DEFAULT_DISPLAY);
    //Initialize egl
    eglInitialize(egldisplay, NULL, NULL);
    assert(eglGetError() == EGL_SUCCESS);
    //tell the driver we are using OpenGL ES
    eglBindAPI(EGL_OPENGL_ES_API);

    //pass our egl configuration to egl
    eglChooseConfig(egldisplay, s_configAttribs, &eglconfig, 1, &numconfigs);
    printf("chooseconfig, \n");
    assert(eglGetError() == EGL_SUCCESS);
    assert(numconfigs == 1);

    //You must pass in the file system handle to the linux framebuffer when creating a window
    #ifndef USE_MX31
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, open("/dev/fb0", O_RDWR), NULL);
    #else
    eglsurface = eglCreateWindowSurface(egldisplay, eglconfig, (NativeWindowType)0, NULL);
    #endif
    printf("createwindow, \n");
    assert(eglGetError() == EGL_SUCCESS);

    #endif

    //create the egl graphics context
    eglcontext = eglCreateContext(egldisplay, eglconfig, NULL, NULL);
    printf("creatcontext, \n");
    assert(eglGetError() == EGL_SUCCESS);

    //make the context current
    eglMakeCurrent(egldisplay, eglsurface, eglsurface, eglcontext);
    printf("makecurrent, \n");
    assert(eglGetError() == EGL_SUCCESS);

    GLfloat diffuse[]={0.5f, 0.5f, 0.5f, 1.0f};
    GLfloat fogColor[]={0.0f, 0.5f, 0.0f, 0.0f};
    GLfloat mat_amb[4]={0.2f, 0.2f, 0.2f, 0.2f};
    GLfloat mat_dif[4]={0.8f, 0.8f, 0.8f, 0.8f};


    glClearColor(0.5f, 0.5f, 0.5f, 1.0f);
    //Do not want to see smoothed colors, only a plain color for face
    glShadeModel(GL_SMOOTH);
    //Enable the depth test in order to see the cube correctly
    glEnable(GL_DEPTH_TEST);

    /*Taking care of specifying correctly the winding order of the
    vertices (counter clock wise order), we can cull all back faces.
    This is probably one of the best optimizations we could do,
    because, by this way, we avoid a lot of computations that wont be
    reflected in the screen. Use  glEnable(GL_CULL_FACE) to do the work*/
    glEnable(GL_CULL_FACE);

    /*set perspective*/
    EGLint h, w;
    /*get width and height from egl*/
    eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &w);
    eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &h);

    /*change to projection matrix*/
    glMatrixMode(GL_PROJECTION);
    /*reset the projection matrix*/
    glLoadIdentity();
    /*set the viewport*/
    glViewport(0, 0, w, h);

    GLUmat4 perspective;
    /*use glu to set perspective*/
    gluPerspective4f(&perspective, 45.0f,((GLfloat)w/(GLfloat)h), 1.0f, 100.0f);
    glMultMatrixf(&perspective.col[0].values[0]);

    /*get back to model view matrix*/
    glMatrixMode(GL_MODELVIEW);

    //Light Settings
    //Set a 50% grey diffuse component
    glLightfv(GL_LIGHT0, GL_DIFFUSE, diffuse);
    //Enable lighting computations
    glEnable(GL_LIGHTING);
    //Enable Light0 (switch on it)
    glEnable(GL_LIGHT0);

    // Set default OpenGL ES material settings
    glMaterialfv(GL_FRONT_AND_BACK, GL_AMBIENT, mat_amb);
    glMaterialfv(GL_FRONT_AND_BACK, GL_DIFFUSE, mat_dif);

    //Fog settings
    //50% green fog
    glFogfv(GL_FOG_COLOR, fogColor);
    /*We chosen a linear mode for fog. Fog will begin to show its effects at 20 units
    of distance from the camera, all the highest fog density will be reached at 50 units*/
    glFogf(GL_FOG_MODE, GL_LINEAR);
    glFogf(GL_FOG_START, 20.0f);
    glFogf(GL_FOG_END,   50.0f);

    //Load our scene file
    mesh = LoadMeshFromFile("shades_imx.gsd");   

    return mesh?1:0;
}

/* Here goes our drawing code */
int render()
{
  static int rotation = 0;

  glClear(GL_COLOR_BUFFER_BIT | GL_DEPTH_BUFFER_BIT);  

  glLoadIdentity();

  /* "camera" setup to have a good point of view of the scene, also we will perform a rotation
     on the mesh */
  glTranslatef(0.0f, -5.0f, -40.0f);
  glRotatef(20.0f, 1.0f, 0.0f, 0.0f);
  glRotatef(rotation++, 0.0f, 1.0f, 0.0f);

  /* With x and z we compute the points of a circle, with a radius = 30, and the velocity = LightRotation */
  static float LightRotation=0.0f;
  float x=0.0f, z=0.0f;
  LightRotation+=0.1f;
  x=30.0f*(float)sin(LightRotation);
  z=30.0f*(float)cos(LightRotation); 
  GLfloat lightPosition[] = { 0.0f, 10.0f, 0.0f, 1.0f };
  lightPosition[0] = x;  lightPosition[2] = z;
  /* Setup of Light0 position */
  glLightfv(GL_LIGHT0, GL_POSITION, lightPosition);

  glEnableClientState(GL_VERTEX_ARRAY);
  glVertexPointer(3, GL_FLOAT, 0, mesh->Geometry);

  /* Normals are needed for the lighting computations */
  glEnableClientState(GL_NORMAL_ARRAY);
  glNormalPointer(GL_FLOAT, 0, mesh->Normals);

  glDrawElements(GL_TRIANGLES,mesh->indexCounter,GL_UNSIGNED_SHORT,mesh->Indices);

  glDisableClientState(GL_VERTEX_ARRAY);
  glDisableClientState(GL_NORMAL_ARRAY);

  return 1;
}

void deinit(void)
{
	//call clean up to release any memory allocated
	Cleanup();
	//Make a empty surface current
	eglMakeCurrent(egldisplay, EGL_NO_SURFACE, EGL_NO_SURFACE, EGL_NO_CONTEXT);
	assert(eglGetError() == EGL_SUCCESS);
	//destroy egl desplay
	eglTerminate(egldisplay);
	assert(eglGetError() == EGL_SUCCESS);
	//end egl thread
	eglReleaseThread();
	#ifdef EGL_USE_X11
	//close x display	
	XCloseDisplay(display);
	#endif
}

int main(int argc, char** argv)
{
    assert( init() );

	while (currentFrame < 1000)
	{
		EGLint width = 0;
		EGLint height = 0;
		eglQuerySurface(egldisplay, eglsurface, EGL_WIDTH, &width);
		eglQuerySurface(egldisplay, eglsurface, EGL_HEIGHT, &height);
		render();
		currentFrame++;
		eglSwapBuffers(egldisplay, eglsurface);
	}
	
    deinit();
    return 1;
}
